﻿using System;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Threading;

using Arction.WPF.LightningChartUltimate;
using Arction.WPF.LightningChartUltimate.Axes;
using Arction.WPF.LightningChartUltimate.SeriesXY;
using Arction.WPF.LightningChartUltimate.Views.ViewXY;

namespace MultipleAsyncChartWindows
{
	internal sealed class ChartWindow : Window
	{
		public delegate void StartedEventHandler();

		public event StartedEventHandler Started;

		private DispatcherTimer m_timer;
		private LightningChartUltimate m_chart;
		private Double m_dLatestX;
		private Double m_dLatestTemperature;
		private Random m_random;

		public ChartWindow()
		{
			m_dLatestX = 0.0;
			m_dLatestTemperature = 50.0;
			m_random = new Random((Int32)DateTime.Now.Ticks);

			// Create and initialize a chart instance.

			m_chart = new LightningChartUltimate("Trial user/LightningChart Ultimate SDK v.6 Trial 2014-2015/LightningChartUltimate/BKDD2QDJYKGNSB32PZADFTCJ9VJREYYX4UTU");

			m_chart.BeginUpdate();

			ViewXY viewXY = m_chart.ViewXY;
			AxisX xAxis = m_chart.ViewXY.XAxes[0];
			AxisY yAxis = viewXY.YAxes[0];

			DateTime now = DateTime.Now;
			Double dMinX = xAxis.DateTimeToAxisValue(now);
			Double dMaxX = xAxis.DateTimeToAxisValue(now) + 30;
			xAxis.SetRange(dMinX, dMaxX);

			xAxis.ValueType = AxisValueType.DateTime;
			xAxis.Title.Text = "Time";
			xAxis.AutoFormatLabels = false;
			xAxis.LabelsTimeFormat = "dd/MM/yyyy\nHH:mm.ss";
			xAxis.LabelsAngle = 90;
			xAxis.MajorGrid.Color = Color.FromArgb(50, 255, 255, 255);

			yAxis.Title.Text = "Temperature / °C";
			yAxis.MajorGrid.Visible = false;
			yAxis.SetRange(0, 100);

			viewXY.Margins = new Thickness(80, 30, 10, 100);
			viewXY.GraphBackground.Color = Colors.DimGray;
			viewXY.GraphBackground.GradientColor = Colors.Black;
			viewXY.GraphBackground.GradientDirection = 270;
			viewXY.GraphBackground.GradientFill = GradientFill.Linear;
			viewXY.DropOldSeriesData = true;
			viewXY.LegendBox.Visible = false;
			viewXY.AxisLayout.AxisGridStrips = XYAxisGridStrips.Y;

			// One point line series for data.
			PointLineSeries series = new PointLineSeries(viewXY, xAxis, yAxis);
			series.LineStyle.Color = Colors.Yellow;
			series.MouseInteraction = false;

			viewXY.PointLineSeries.Add(series);

			m_chart.EndUpdate();

			// Add the chart and a "sleeper" button to a grid.

			Button button = new Button();
			button.Content = "Sleep for 5 seconds";
			button.Click +=
				new RoutedEventHandler(
					delegate(Object sender, RoutedEventArgs e)
					{
						Thread.Sleep(5000);
					}
				);

			Grid grid = new Grid();
			grid.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(0.5, GridUnitType.Star) });
			grid.RowDefinitions.Add(new RowDefinition() { Height = GridLength.Auto });
			grid.Children.Add(m_chart);
			grid.Children.Add(button);

			Grid.SetRow(m_chart, 0);
			Grid.SetRow(button, 1);

			// The grid is set as window's content.
			Content = grid;

			// Subscribe to Closed event so that we know when window is closed.
			Closed += new EventHandler(ChartWindow_Closed);

			// Set window size.
			Height = 480;
			Width = 640;

			// Create and initialize a timer for data generating.
			m_timer = new DispatcherTimer();
			m_timer.Interval = new TimeSpan(0, 0, 0, 0, 10);
			m_timer.Tick += m_timer_TickFirst;
			m_timer.Start();
		}

		internal LightningChartUltimate Chart
		{
			get
			{
				return m_chart;
			}
		}

		private void ChartWindow_Closed(object sender, EventArgs e)
		{
			// Stop the timer.
			m_timer.Stop();

			// When the window is closed, we need to 
			// dispose of the chart so that its unmanaged 
			// resources do not stay in memory as garbage.
			m_chart.Dispose();

			// Then we need to call Dispatcher.BeginInvokeShutdown 
			// method so that the window's dispatcher is shut down. 
			// Otherwise it would run in the background preventing 
			// the application from shutting down.
			Dispatcher.BeginInvokeShutdown(DispatcherPriority.Background);
		}

		private void m_timer_TickFirst(object sender, EventArgs e)
		{
			if (Started != null)
			{
				Started();
			}

			m_timer_Tick(sender, e);

			m_timer.Tick -= m_timer_TickFirst;
			m_timer.Tick += m_timer_Tick;
		}

		private void m_timer_Tick(object sender, EventArgs e)
		{
			m_dLatestX = m_chart.ViewXY.XAxes[0].DateTimeToAxisValue(DateTime.Now);
			m_dLatestTemperature = CalculateYValue();

			SeriesPoint[] points = new SeriesPoint[1];
			points[0].X = m_dLatestX;
			points[0].Y = m_dLatestTemperature;

			m_chart.BeginUpdate();
			m_chart.ViewXY.PointLineSeries[0].AddPoints(points, false);
			m_chart.ViewXY.XAxes[0].ScrollPosition = m_dLatestX;
			m_chart.EndUpdate();
		}

		private Double CalculateYValue()
		{
			Double dY = m_dLatestTemperature + (m_random.NextDouble() - 0.5) * 1.0;
			if (dY > 100)
			{
				dY = 100;
			}
			else if (dY < 0)
			{
				dY = 0;
			}

			return dY;
		}
	}
}
