﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Windows;
using System.Windows.Media;
using System.Windows.Threading;
using System.Windows.Controls;

using Arction.WPF.LightningChartUltimate;
using Arction.WPF.LightningChartUltimate.EventMarkers;
using Arction.WPF.LightningChartUltimate.Titles;

namespace MultipleAsyncChartWindows
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml.
	/// </summary>
	public partial class MainWindow : Window
	{
		private List<ChartWindow> m_chartContainers;
		private Int32 m_iChartWindowsToCreateAtStart;

		public MainWindow()
		{
			m_chartContainers = new List<ChartWindow>();
			m_iChartWindowsToCreateAtStart = 3;

			InitializeComponent();

			CreateNewChartWindow();
		}

		private void CreateNewChartWindow()
		{
			Thread newWindowThread =
				new Thread(
					new ThreadStart(
						delegate
						{
							// Create a new ChartWindow instance from a thread. 
							// This way it is not owned by the main UI thread.
							ChartWindow cc = new ChartWindow();
							cc.Closed += new EventHandler(cc_Closed);
							cc.Started += new ChartWindow.StartedEventHandler(cc_Started);
							cc.Show();

							// Add the chart window to the list so that we can access it 
							// later.
							m_chartContainers.Add(cc);

							// Call Dispatcher.Run method and the chart window will 
							// have its own dispatcher ("UI thread").
							Dispatcher.Run();		
						}
					)
				);

			newWindowThread.SetApartmentState(ApartmentState.STA);
			newWindowThread.Start();
		}

		private void cc_Started()
		{
			if (m_iChartWindowsToCreateAtStart > 0)
			{
				m_iChartWindowsToCreateAtStart--;

				CreateNewChartWindow();
			}
		}

		private void cc_Closed(object sender, EventArgs e)
		{
			// Remove the chart window from the list.
			m_chartContainers.Remove(sender as ChartWindow);
		}

		private void buttonOpen_Click(object sender, RoutedEventArgs e)
		{
			CreateNewChartWindow();
		}

		private void buttonSleep_Click(object sender, RoutedEventArgs e)
		{
			Thread.Sleep(5000);
		}

		private void buttonMarker_Click(object sender, RoutedEventArgs e)
		{
			// Let's use the same timestamp for every chart.
			DateTime now = DateTime.Now;

			foreach (ChartWindow chartContainer in m_chartContainers)
			{
				// Let's use ThreadPool to add a marker to each chart. 
				// This way the "main window" stays resposive if one of the 
				// chart windows is doing something that blocks its 
				// dispatcher (e.g. Thread.Sleep is called). Marker is 
				// added as soon as it's possible with the same timestamp 
				// as with the other chart windows.

				ThreadPool.QueueUserWorkItem(
					new WaitCallback(
						delegate (Object state)
						{
							ChartWindow cc = state as ChartWindow;							
							cc.Dispatcher.Invoke(
								new Action(
									delegate
									{
										LightningChartUltimate chart = cc.Chart;
										Color markerColor = Colors.Red;
										PointShapeStyle shape =
											new PointShapeStyle(
												null,
												Arction.WPF.LightningChartUltimate.Shape.Triangle,
												Color.FromArgb(0, markerColor.R, markerColor.G, markerColor.B),
												markerColor,
												Colors.White,
												Colors.Black,
												12.0f,
												12.0f,
												0.0f,
												180.0f,
												GradientFillPoint.Edge,
												Direction.Right,
												null
											);

										EventMarkerTitle title = new EventMarkerTitle(null);
										title.Text = (chart.ViewXY.PointLineSeries[0].SeriesEventMarkers.Count + 1).ToString();
										title.Color = System.Windows.Media.Color.FromArgb(128, 255, 255, 255);
										title.Font = new WPFFont(System.Drawing.FontFamily.GenericSansSerif, 8.0, System.Drawing.FontStyle.Bold);

										SeriesEventMarker marker =
											new SeriesEventMarker(
												chart.ViewXY.PointLineSeries[0],
												shape,
												SeriesEventMarkerVerticalPosition.YAxisTop,
												chart.ViewXY.XAxes[0].DateTimeToAxisValue(now),
												0.0,
												title
											);
										marker.HorizontalPosition = SeriesEventMarkerHorizontalPosition.AtXValue;
										marker.Offset = new PointIntXY(0, (Int32)(shape.Height / 2.0f));
										
										chart.ViewXY.PointLineSeries[0].SeriesEventMarkers.Add(marker);
									}
								)
							);
						}
					),
					chartContainer as Object
				);
			}
		}
	}
}
